// 
// Glow Hott pour télémesure Graupner Hott
// 
// Version 1.11
// Avril 2015
//
// Auteur : www.thyzoon.fr 
// 2014-2015
// 
//

//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
//****************//    
//-- PIN Mapping -//
//
// D0 : RXD Réception Hott
// D1 : TXD Emission Hott
// D2 : Entrée signal voie gaz. (RX)
// D3 : Entrée signal capteur pour compte tour hélice
// D4 : LED securité
// D5 : LED chauffage
// D10: Sortie commande MOSFET de chauffage 
// A0 : Tension NIMH 
// A1 : Tension mesure temperature (RCTN)
// A2 : Tension LIFE ou LIPO
//****************** 
    
#include <EEPROM.h>
#include "Message.h"
#include "Mesure.h"

// ---------------
// fonctions
void Init_rx();  // Initialisation des interruptions RX gaz
void Init_ctr(); // Initialisation des interruptions compte tours
word median(word t); // filtre

// Def LED verte carte Arduino mini pro
#define LEDPIN_PINMODE        pinMode (13, OUTPUT);
#define LEDPIN_SWITCH         PINB |= 1<<5;     //switch LEDPIN state (digital PIN 13)
#define LEDPIN_OFF            PORTB &= ~(1<<5);
#define LEDPIN_ON             PORTB |= (1<<5);

// LED securité (4)
#define LEDSECU_MODE          pinMode (4, OUTPUT);
#define LEDSECU_OFF           PORTD &= ~(1<<4);
#define LEDSECU_ON            PORTD |= (1<<4);

// LED chauffage (5)
#define LEDCHAU_MODE          pinMode (5, OUTPUT);
#define LEDCHAU_OFF           PORTD &= ~(1<<5);
#define LEDCHAU_ON            PORTD |= (1<<5);

// Coupure moteur et remise en sécurité
#define DELAI_MISE_SECU       1500  // (ms) délai avant retour en sécurité

// Son des alarmes 
#define ALARME_CARBURANT        0x08
#define ALARME_TEMPERATURE      0x15

byte bip_alarme;                // son de l'alarme

// Baudrate UART pour Graupner Hott
#define SERIAL_COM_SPEED 19200

//byte test;

// ++++++++++++++++++++++++++++++++
// Gestion entrée RX
volatile word tpulse;     // Durée impulsion RX 
word tpulse_0;   // Durée impulsion RX precedente
word tpulse_1;   // Durée impulsion RX courante
boolean flag_pulse_rx;    // Nouvelle impulsion RX
word position_gaz;        // Position courante de commande des gaz
int variation;            // variation de largeur d'impulsion (µs) entre 2 impulsions RX

// Variables parametres EEPROM
extern word ralenti_gaz;       // seuil ralenti commande des gaz pour chauffage
extern word coupure_gaz;       // seuil coupure commande des gaz pour chauffage
extern word pos_armement;      // armement. à la mise sous tension position sécu 
extern boolean gaz_inverse;    // sens des gaz inversé (ou normal)
extern word delai;             // délai de temporisation de chauffage bougie
extern word nb_pales;          // nombre de pales de l'hélice
extern word courant_dem;       // Courant de chauffage démarrage
extern word courant_trn;       // Courant de chauffage moteur tournant
extern word vitesse_gaz;       // Vitesse de déplacement du manche des gaz déclenchant la chauffage de la bougie
extern word regime_moteur_maxi; // Régime moteur maxi du compte-tours
extern word alarme_moteur;      // temps alarme moteur
extern word alarme_repet;       // temps de repetition alarme
extern word alarme_temperature; // temperature d'alarme moteur
extern word alarme_on;          // alarme on

word pos_coupure;       // remise en securité (position gaz)
boolean securite;       // gestion de la securite
boolean tempo_secu;     // gestion de la temporisation de remise en sécurité
unsigned long temps_secu;

// Variables de commande de chauffage
#define PWM_PIN 10 
#define PWM_STOP 0
#define PWM_FULL 255

boolean cmd_chauffe;          // Commande de chauffage 
byte pwm_chauffe;             // Commande de chauffage en PWM de 0 à 255
byte pwm_affichage_temp;      // PWM chauffage pour affichage sur telemesure Hott
unsigned long timer_chauffe;  // temps pour gestion de la tempo chauffage
unsigned long temps_ms;       // temps en ms

// Gestion état du moteur et comptage temps moteur en marche
boolean moteur_marche;        // moteur en marche
unsigned long Compteur_temps_moteur; // en ms
word Temps_moteur;  // en s 
unsigned long Temps_courant; // temp en ms
unsigned long Temps_marche_moteur;   // temps  en ms

boolean Alarme_carb;   // Alarme carburant (temps moteur atteint)
word temps_alarme;     // temps prochaine alarme
int Temperature_capteur; // Temperature du moteur

// Mesure compte tours
#define MOT_MIN 1800       // regime min mesure = 1800 tr/min

volatile unsigned long temps_helice;  // temps passages de pales en µs
volatile boolean flag_temps_helice;   // nouveau temps hélice valide
word trmn_helice;                     // tours par minutes
word trmn_helice_val;                 // tours par minutes 

// Pour detection de moteur à l'arrêt
unsigned long tps_a_ms;        // temps en ms
unsigned long tps_b_ms;        // temps en ms


// Gestion messagerie Hott
GMessage message;

// Gestion des mesure de tension
Mesure analogique;

//********************************************
// Initialisations 
//
void setup() {
  // Init communications 
  message.init();
  analogique.init();
  
  // LED carte
  LEDPIN_PINMODE
  LEDPIN_OFF 
  
  // LED secu
  LEDSECU_MODE
  LEDSECU_ON
  
  // LED chauffage bougie
  LEDCHAU_MODE
  LEDCHAU_ON

  delay (20);
  LEDCHAU_OFF
  LEDSECU_ON   
  
  // Init entrée RX gaz 
  if (gaz_inverse == true) {
    tpulse_0 = 1900;
    tpulse_1 = 1900;
    tpulse = 1900;
  }
  else {
    tpulse_0 = 1100;
    tpulse_1 = 1100; 
    tpulse = 1100;
  }
  position_gaz = 1100;
  variation = 0;
  
  // Init variables compte-tours
  trmn_helice = 0;
  trmn_helice_val = 0;  
  tps_a_ms =0;
  tps_b_ms =0;
  moteur_marche = false;
 
  // Init variables temps moteur en fonctionnement
  Compteur_temps_moteur = 0;
  Temps_moteur = 0; 
  Temps_courant = 0;  
  Temps_marche_moteur = 0; 
  
  timer_chauffe = 0;
  securite = true;     // en securité à la mise sous tension
  tempo_secu = false;
  
  // Sortie PWM chauffage = arrêt
  analogWrite(PWM_PIN, PWM_STOP);
  cmd_chauffe = false;
  pwm_affichage_temp = 0;
  pwm_chauffe = 0;
 
  pos_coupure = coupure_gaz;
  
  // Gestion alarmes
  Alarme_carb = false;  // Init alarme
  temps_alarme = alarme_moteur;
  bip_alarme = 0;       
  
  // Init entrée RX gaz et interruptions  
  Init_rx (); 
  flag_pulse_rx = false;
  
  // Init entrée compte tours hélice et interruptions  
  flag_temps_helice = false;
  temps_helice = 0;
  
  Init_ctr ();   
}

//******************************************
// Boucle pricipale
//
void loop() {
    
  //*********************************************
  // Gestion des impulsion RX
  //
  if (flag_pulse_rx == true) { // Mesure RX
    flag_pulse_rx = false;
    
      // Controle de la largeur d'impulsion en µs 900 -- 2100 
      if ((tpulse >= 900) && (tpulse <= 2100 )) {
        tpulse_0 = tpulse_1;      // memo durée pulse précédente
        tpulse_1 = tpulse;        // pulse courante
       
       // -------------------------------------------
       // prise en compte du sens de commande des gaz 
       // Excursion = 0 à 3000µs; course utile; centre à 1500µs
       // Gaz normal  ==> 900µs à 2100µs
       // Gaz inversé ==> 2100µs à 900µs 
       // Remarque : 1 cran de trim = 5µs 
       if (gaz_inverse == true) {
         position_gaz = 3000 - tpulse_1;  // changement de "sens"
       } 
       else {
         position_gaz = tpulse_1;  
       }       
       
       //---------------------------------------------
       // calcul de la variation entre 2 impulsions RX
       // tpulse_1 et tpulse_0 provenant de la capture 
       // sous interruption (largeur impulsion en µs)
       variation = tpulse_1 - tpulse_0; // en µs
       variation = abs(variation);            
    }
  }

  // ---------------------------------------------
  // Gestion securité et armement
  // Armement = manche gaz en haut 
  if ((securite == true) && (position_gaz >= pos_armement)) {
    securite = false; // armé
    LEDSECU_OFF
  }
  // Securité = coupure moteur  
  // Temporisation pour prise en compte de la remise en sécurité
  if ((securite == false) && (position_gaz <= pos_coupure)) {  
    if (tempo_secu == false) {
      tempo_secu = true;
      temps_secu = millis();  // temps de début 
    }
    else {
      temps_ms = millis();  // temps courant
      temps_ms = temps_ms - temps_secu; 
      if (temps_ms > DELAI_MISE_SECU) { 
        securite = true;  // en securité 
        LEDSECU_ON 
      }
    }  
  }
  else {
    tempo_secu = false;
  }
  
  //--------------------------------------------
  // Gestion de la sortie PWM de commande de chauffage
  // Conditions :
  // - Ralenti : seuil coupure_moteur < posistion_gaz < ralenti_gaz
  // - Sortie de ralenti : temporisation de xx ms
  // - Variation de gaz rapide
  
  // ralenti_gaz : seuil de chauffage au ralenti
  // coupure_gaz : seuil de coupure du moteur
  // au ralenti le chauffage est réalisé entre "coupure_gaz" et "ralenti_gaz"
  
  if ((position_gaz >= coupure_gaz) && (position_gaz <= ralenti_gaz)) {
    cmd_chauffe = true;  
    if (moteur_marche == false) 
      pwm_chauffe = map(courant_dem, 0, 100, PWM_STOP, PWM_FULL);     // Chauffage position ralenti moteur à l'arrêt
    else  
      pwm_chauffe = map(courant_trn, 0, 100, PWM_STOP, PWM_FULL);     // Chauffage position ralenti moteur en marche      
    timer_chauffe = millis();   // temps courant depuis alimentation en ms
  }

  //--------------------------------------------- 
  // Detection de vitesse de variation de manche de Gaz
  // en valeur en (µs)
  if ((variation > vitesse_gaz) && (position_gaz >= coupure_gaz)) {
    cmd_chauffe = true;  
    pwm_chauffe = map(courant_trn, 0, 100, PWM_STOP, PWM_FULL);    // Chauffage en variation rapide manche gaz
    timer_chauffe = millis();   // temps courant depuis alimentation en ms    
  }  

  // -----------------------------
  // Commande chauffage bougie
  // 
  if ((cmd_chauffe == true) && (securite == false)){ // Gestion avec securite
    LEDCHAU_ON
    analogWrite(PWM_PIN, pwm_chauffe); // Commande PWM MOSFET , courant bougie
    pwm_affichage_temp = (byte) ((float) pwm_chauffe / 2.55);  // pour affichage Hott sur Temp2
    temps_ms = millis();   // temps courant depuis alimentation bougie en ms    
    // Fin de temporisation
    if ((temps_ms > (timer_chauffe + delai)) &&  (tempo_secu == false)) // Coupure après tempo sauf si secu en cours
      cmd_chauffe = false;  // coupure 
  }
  else {
    // coupure alimentation en courant de la bougie
    pwm_chauffe = PWM_STOP;
    analogWrite(PWM_PIN, pwm_chauffe); 
    pwm_affichage_temp = 0; 
    LEDCHAU_OFF
  }  

  // ---------------------------------------------
  // Régime moteur
  // Traitement temps hélice
  // 
  // Le fitrage est mis en place afin de traiter les fausses détections qui se produisent avec un capteur optique
  //
  if (flag_temps_helice == true) {
    LEDPIN_ON
    flag_temps_helice = false;	
    // calcul tours / minutes
    trmn_helice = 60000000 / temps_helice / nb_pales;       // 60 (s) x 10e6 = 60000000 (60s par minutes et temps_helice en µs) 
    if ((trmn_helice > MOT_MIN) && (trmn_helice < (regime_moteur_maxi*10))) { 
      // Valeur median pour filtrage mauvaise mesure (necessaire avec le capteur optique) 
      trmn_helice_val = median (trmn_helice);      // Filtrage compte-tours    
      moteur_marche = true;
      tps_a_ms = millis(); // temps courant moteur tournant
    }
    LEDPIN_OFF
  }
  else {
    // Detection d'arret moteur
    tps_b_ms = millis(); // temps courant
    if ((tps_b_ms - tps_a_ms) > 100){   // 100 ms sans temps hélice = arret moteur detecté 
      trmn_helice_val = 0;
      moteur_marche = false;
    } 
  }  
  
 // Gestion du chronometre temps moteur pour gestion de l'alarme sur temps moteur tournant 
 if (moteur_marche == true)
 {
    Temps_courant = millis();  // temps courant ms
    Compteur_temps_moteur = Compteur_temps_moteur + (Temps_courant - Temps_marche_moteur);
    Temps_moteur = Compteur_temps_moteur / 1000; //  en secondes  
    Temps_marche_moteur = Temps_courant;  
 }  
   
  // Alarme depassement du temps moteur 
  // Temps maxi atteint 
  if (Temps_moteur > temps_alarme)
  {
    temps_alarme = temps_alarme + alarme_repet; 
    bip_alarme = ALARME_CARBURANT;
  }   
 
  // Alarme temperature moteur    
  Temperature_capteur = analogique.Temperature ();
  if ((Temperature_capteur > alarme_temperature) && (bip_alarme != ALARME_CARBURANT)) 
  // Pas d'alarme temps moteur en cours
      bip_alarme = ALARME_TEMPERATURE;
    
  // Gestion des message Hott
  message.main();

  // ---------------------------------------------
  //Serial.println(temps_int);
}
